
import es
import os
import time
import socket
import urllib
import zipfile
import threading
import servsecurity
import config_rewrite

UpdateCheckURL   = "http://servsecurity.com/servsecurity/0.9/updatecheck.php"
CurrentVerURL    = "http://servsecurity.com/servsecurity/0.9/updates/current_ver.zip"
ZIPVerification  = "http://85.114.130.27/servsecurity/0.9/hash.php"
UpdateInProgress = False

def Check4Updates():
    threading.Thread(target=UpdateCheckThreadProxy, args=()).start()

def ForceUpdate():
    es.server.printmsg("[ServSecurity] Force update...")
    UpdateCheckThreadProxy()

def UpdateLog(Text):
    fp = open(es.getAddonPath('servsecurity') + os.sep + "UpdateLog.log", 'a')
    fp.write(Text + os.linesep)
    fp.close()

def UpdateCheckThreadProxy():
    global UpdateInProgress
    if UpdateInProgress:
        return

    UpdateInProgress = True
    UpdateCheckThread()
    UpdateInProgress = False

def UpdateCheckThread():
    servsecurity.Log(3, "Check for updates...")
    socket.setdefaulttimeout(5)

    try:
        Obj = urllib.urlopen(UpdateCheckURL)
        Result = Obj.read()
        Obj.close()
    except:
        UpdateLog("FAIL: Cant get current version")
        return False

    Results = Result.split("|")
    Result_Version = Results[0]
    Result_Build = int(Results[1])

    if Result_Build > servsecurity.info.build:
        servsecurity.Log(1, "New ServSecurity version available")
        UpdateLog("STATUS: New version found (" + Result_Version + ")")
        if int(es.ServerVar('servsecurity_autoupdate')):
            MD5 = False
            UpdateLog("STATUS: Downloading verification hash...")
            try:
                Obj = urllib.urlopen(ZIPVerification)
                Result = Obj.read()
                Obj.close()
                if len(Result) == 32:
                    MD5 = Result
            except:
                UpdateLog("FAIL: Cant get verification hash, ignoring...")

            UpdateLog("STATUS: Downloading ZIP...")
            try:
                Path = es.ServerVar('eventscripts_gamedir') + os.sep + "ServSecurityUpdate.zip"
                if os.path.isfile(Path):
                    UpdateLog("STATUS: Removing old update file...")
                    os.remove(Path)

                Obj = urllib.urlopen(CurrentVerURL)
                fp = open(Path, 'wb')
                fp.write(Obj.read())
                Obj.close()
                fp.close()

                if MD5:
                    FileMD5 = servsecurity.md5_file(Path)
                    if MD5 != FileMD5:
                        os.remove(Path)
                        UpdateLog("FAIL: Hashcheck! %s != %s Abort..." % (MD5, FileMD5))
                        return False
            except:
                UpdateLog("FAIL: Downloading failed! Abort...")
                return False

            time.sleep(5)

            PassWD = str(servsecurity._Password)
            servsecurity.Log(1, "Auto-Update...")

            Dir = es.ServerVar('eventscripts_gamedir') + os.sep
            try:
                UpdateLog("STATUS: Unpacking ZIP-File...")
                zfobj = zipfile.ZipFile(Path)
                for name in zfobj.namelist():
                    FilePath = os.path.join(Dir, name)
                    if FilePath.endswith('/'):
                        if not os.path.exists(FilePath):
                            os.mkdir(FilePath)
                    else:
                        if os.path.exists(FilePath):
                            UpdateLog("STATUS: Remove " + FilePath)
                            os.chmod(FilePath, 0777)
                            os.remove(FilePath)
                        UpdateLog("STATUS: Write '" + FilePath + "'...")
                        outfile = open(FilePath, 'wb')
                        outfile.write(zfobj.read(name))
                        outfile.close()

                time.sleep(1)
                UpdateLog("STATUS: Write config...")
                if not config_rewrite.ReWriteConfig(PassWD):
                    UpdateLog("FAIL: config_rewrite.ReWriteConfig(). Abort...")
                time.sleep(1)

                try:
                    os.remove(es.ServerVar('eventscripts_gamedir') + os.sep + "ServSecurityUpdate.zip")
                except:
                    pass

                UpdateLog("UPDATE SUCCESSFUL! THANKS FOR USING SERVSECURITY!")
                servsecurity.info.build = Result_Build
                servsecurity._AllowUnload = True
                es.unload('servsecurity')
                time.sleep(2)
                UpdateLog("STATUS: Loading ServSecurity...")
                es.server.queuecmd("es_xreload servsecurity")
            except:
                UpdateLog("FATAL ERROR! Update aborted!")
                es.server.insertcmd("es_xload servsecurity")
                try:
                    os.remove(es.ServerVar('eventscripts_gamedir') + os.sep + "ServSecurityUpdate.zip")
                except:
                    pass
    else:
        servsecurity.Log(3, "ServSecurity up-to-date!")